package sun.security.provider.certpath;

import java.util.ListIterator;
import java.security.cert.CertificateException;
import sun.security.x509.SubjectAlternativeNameExtension;
import java.io.IOException;
import sun.security.x509.GeneralName;
import sun.security.x509.X500Name;
import java.security.cert.X509Certificate;
import java.security.cert.CertPathValidatorException;
import java.util.Iterator;
import java.util.List;
import java.security.cert.PKIXCertPathChecker;
import java.util.ArrayList;
import sun.security.x509.GeneralNameInterface;
import java.util.HashSet;
import sun.security.x509.X509CertImpl;
import javax.security.auth.x500.X500Principal;
import sun.security.util.Debug;

class ForwardState implements State
{
    private static final Debug debug;
    X500Principal issuerDN;
    X509CertImpl cert;
    HashSet<GeneralNameInterface> subjectNamesTraversed;
    int traversedCACerts;
    private boolean init;
    UntrustedChecker untrustedChecker;
    ArrayList<PKIXCertPathChecker> forwardCheckers;
    boolean keyParamsNeededFlag;
    
    ForwardState() {
        this.init = true;
        this.keyParamsNeededFlag = false;
    }
    
    @Override
    public boolean isInitial() {
        return this.init;
    }
    
    @Override
    public boolean keyParamsNeeded() {
        return this.keyParamsNeededFlag;
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("State [");
        sb.append("\n  issuerDN of last cert: ").append(this.issuerDN);
        sb.append("\n  traversedCACerts: ").append(this.traversedCACerts);
        sb.append("\n  init: ").append(String.valueOf(this.init));
        sb.append("\n  keyParamsNeeded: ").append(String.valueOf(this.keyParamsNeededFlag));
        sb.append("\n  subjectNamesTraversed: \n").append(this.subjectNamesTraversed);
        sb.append("]\n");
        return sb.toString();
    }
    
    public void initState(List<PKIXCertPathChecker> list) throws CertPathValidatorException {
        this.subjectNamesTraversed = new HashSet();
        this.traversedCACerts = 0;
        this.forwardCheckers = new ArrayList();
        for (PKIXCertPathChecker e : list) {
            if (e.isForwardCheckingSupported()) {
                e.init(true);
                this.forwardCheckers.add(e);
            }
            else {
                continue;
            }
        }
        this.init = true;
    }
    
    @Override
    public void updateState(X509Certificate cert) throws CertificateException, IOException, CertPathValidatorException {
        if (cert == null) {
            return;
        }
        X509CertImpl impl = X509CertImpl.toImpl(cert);
        if (PKIX.isDSAPublicKeyWithoutParams(impl.getPublicKey())) {
            this.keyParamsNeededFlag = true;
        }
        this.cert = impl;
        this.issuerDN = cert.getIssuerX500Principal();
        if (!X509CertImpl.isSelfIssued(cert) && !this.init && cert.getBasicConstraints() != -1) {
            ++this.traversedCACerts;
        }
        if (this.init || !X509CertImpl.isSelfIssued(cert)) {
            this.subjectNamesTraversed.add(X500Name.asX500Name(cert.getSubjectX500Principal()));
            try {
                SubjectAlternativeNameExtension subjectAlternativeNameExtension = impl.getSubjectAlternativeNameExtension();
                if (subjectAlternativeNameExtension != null) {
                    Iterator<GeneralName> iterator = subjectAlternativeNameExtension.get("subject_name").names().iterator();
                    while (iterator.hasNext()) {
                        this.subjectNamesTraversed.add(((GeneralName)iterator.next()).getName());
                    }
                }
            }
            catch (IOException cause) {
                if (ForwardState.debug != null) {
                    ForwardState.debug.println("ForwardState.updateState() unexpected exception");
                    cause.printStackTrace();
                }
                throw new CertPathValidatorException(cause);
            }
        }
        this.init = false;
        return;
    }
    
    @Override
    public Object clone() {
        try {
            ForwardState forwardState = (ForwardState)super.clone();
            forwardState.forwardCheckers = (ArrayList)this.forwardCheckers.clone();
            ListIterator<Object> listIterator = forwardState.forwardCheckers.listIterator();
            while (listIterator.hasNext()) {
                PKIXCertPathChecker pkixCertPathChecker = (PKIXCertPathChecker)listIterator.next();
                if ((pkixCertPathChecker instanceof Cloneable)) {
                    listIterator.set((Object)pkixCertPathChecker.clone());
                }
                else {
                    continue;
                }
            }
            forwardState.subjectNamesTraversed = (HashSet)this.subjectNamesTraversed.clone();
            return forwardState;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause.toString(), cause);
        }
    }
    
    static {
        debug = Debug.getInstance("certpath");
    }
}
